@@ -7,33 +7,26 @@ import time
 import logging
 import bisect
 import datetime
-import uuid
-import copy
 from typing import List, Union, Iterable
 from .timepoint import ActionTimePoint
 from .action import Action, ConfiguredAction
-from .sequencing import ActionList
+from .sequencing import ConfiguredSequence
+from .mixin import InstanceRegistry
 
 logger = logging.getLogger(__name__)
 
 
-class SamplingScheduler(object):
+class SamplingScheduler(InstanceRegistry):
     # pause time between cycle checks
     CYCLE_TIME = 0.5
 
     # default state for whether to wait for completion of previous trigger if busy
     WAIT_IF_BUSY = True
 
-    # list of active instances
-    instances = []
-
     UUID_PREFIX = 'sched-'
 
-    # class-level list of default actions
-    default_actions: ActionList = ActionList()
-
     def __init__(self,
-                 *actions: Union[str, callable, Action],
+                 *actions: Union[str, callable, Action, ConfiguredAction],
                  time_deltas: List[float] = None,
                  sequence_name: str = None,
                  ):
@@ -45,21 +38,18 @@ class SamplingScheduler(object):
         :param actions: callable action or the name of a callable action previously defined as a SchedulerAction
         :param sequence_name: convenience reference name for the current experiment
         """
-        # unique identifier for instance
-        self._uuid = str(uuid.uuid4())
+        InstanceRegistry.__init__(self)
         logger.debug(f'creating scheduler {self._uuid[:5]}')
         self.logger = logger.getChild(f'{self.__class__.__name__} {self._uuid[:5]}')
 
         # event control of killswitch for stop command
         self._killswitch = threading.Event()
 
-        # store default actions
-        if len(actions) > 0:
-            # todo consider whether this should override or extend
-            self.instance_default_actions: ActionList = ActionList(*actions)
-        else:
-            # todo consider how to extend default list
-            self.instance_default_actions: ActionList = copy.copy(self.default_actions)
+        # create sequence
+        self.sequence: ConfiguredSequence = ConfiguredSequence(
+            *actions,
+            sequence_name=sequence_name,
+        )
 
         # time stamp for Scheduler creation
         self.created = time.time()
@@ -81,8 +71,6 @@ class SamplingScheduler(object):
         # store experiment name for reference
         self.sequence_name: str = sequence_name
 
-        # append instance
-        self.instances.append(self)
         # todo implement databasing
         #   - add method which reinstantiates from database
         # todo have the experiment be named based on the sample number in the sequence
@@ -174,32 +162,18 @@ class SamplingScheduler(object):
             return True
         return False
 
-    @classmethod
-    def get_instance_by_uuid(cls, uuid: str) -> "SamplingScheduler":
-        """
-        Get a sampler instance by UUID
-
-        :param uuid: unique id (expected to be uuid4)
-        :return: sampling scheduler instance
-        """
-        for instance in cls.instances:
-            if instance.uuid == uuid:
-                return instance
-        # todo consider supporting concatenated uuids
-        raise KeyError(f'The uuid "{uuid}" does not match any SamplingScheduler instances.')
-
     @classmethod
     def get_newest_instance(cls) -> "SamplingScheduler":
         """Gets the newest instance of the sampling scheduler"""
         try:
-            return cls.instances[-1]
+            return cls.class_registry[-1]
         except IndexError:
             raise IndexError(f'The {cls.__name__} has no instances. ')
 
     def _determine_actions(self,
                            additional_actions: Iterable[Union[str, callable, ActionTimePoint]] = None,
                            specified_actions: Iterable[Union[str, callable, ActionTimePoint]] = None,
-                           ) -> ActionList:
+                           ) -> ConfiguredSequence:
         """
         Determines the actions for a time point. If no actions are provided, the default actions of the
         instance are used. If additional actions are provided, those actions will be performed in addition to the
@@ -213,12 +187,12 @@ class SamplingScheduler(object):
         """
         # establish list of actions
         if additional_actions is not None:
-            actions = copy.copy(self.instance_default_actions)
-            actions.extend(additional_actions)
+            actions = self.sequence.duplicate_configuration()
+            actions.action_sequence.extend(additional_actions)
         elif specified_actions is not None:
-            actions = ActionList(*specified_actions)
+            actions = ConfiguredSequence(*specified_actions)
         else:
-            actions = copy.copy(self.instance_default_actions)
+            actions = self.sequence.duplicate_configuration()
         return actions
 
     def trigger(self,
@@ -247,7 +221,7 @@ class SamplingScheduler(object):
             )
             self.wait_for_time_point_completion()
 
-        actions = self._determine_actions(
+        action_sequence = self._determine_actions(
             additional_actions=additional_actions,
             specified_actions=specified_actions,
         )
@@ -255,7 +229,7 @@ class SamplingScheduler(object):
         # create a manual time point
         manual_time_point = ActionTimePoint(
             time.time() - self._start_time if self._start_time is not None else 0.,
-            actions=actions,
+            configuration=action_sequence,
         )
         # store start time if set
         if self._start_time is not None:
@@ -337,7 +311,7 @@ class SamplingScheduler(object):
         :param shift_subsequent: whether to shift subsequent time points in the time point list by the time delta
         :param number_of_points: number of points with these specification to add
         """
-        actions = self._determine_actions(
+        action_sequence = self._determine_actions(
             additional_actions=additional_actions,
             specified_actions=specified_actions,
         )
@@ -363,11 +337,13 @@ class SamplingScheduler(object):
         # add time points
         self.logger.debug(f'Adding {number_of_points} timepoints at {time_delta} s spacing in position {position}.')
         for i in range(number_of_points):
+            delta = (i + 1) * time_delta + delta_offset
             bisect.insort(
                 self._time_points,
                 ActionTimePoint(
-                    (i + 1) * time_delta + delta_offset,
-                    actions=actions,
+                    delta,
+                    configuration=action_sequence,
+                    name=f'{delta} s',
                 )
             )
 
@@ -469,6 +445,11 @@ class SamplingScheduler(object):
                         ]
             self._sequence_thread.start()
 
+    @property
+    def paused(self) -> bool:
+        """whether the sequence is paused"""
+        return not self._sequence_pause.is_set()
+
     def pause_sequence(self):
         """Pauses the sequence (actions will not be automatically triggered until resumed)"""
         self._sequence_pause.clear()
@@ -493,8 +474,7 @@ class SamplingScheduler(object):
         out = {
             'uuid': self.uuid,
             'sequence_name': self.sequence_name,
-            'instance_default_actions': [action.name for action in self.instance_default_actions],
-            'class_default_actions': [action.name for action in self.default_actions],
+            'instance_actions': [action.name for action in self.sequence.action_sequence],
             'started': self.started,
             'created': self.created,
             'num_time_points': len(self.time_points),
@@ -520,7 +500,7 @@ class SamplingScheduler(object):
         if key is None:
             tp = self.last_triggered_time_point
         else:
-            tp = ActionTimePoint.get_instance(key)
+            tp = ActionTimePoint.class_instance_by_id(key)
         return tp.action_returns
 
     def visualize_timepoints(self):
