@@ -4,23 +4,20 @@ and `ActionTimePoint` manages context, sequences of actions, and execution of th
 """
 import datetime
 import uuid
-import threading
 import logging
-import time
-from typing import Union, List, Dict, Callable
-from .action import Action, ConfiguredAction
-from .sequencing import ActionList, ConfiguredActionList, TrackedActionList
-
+import warnings
+from typing import Union, Callable, Iterable
+from .mixin import InstanceRegistry
+from .action import Action, ConfiguredAction, ConfiguredActionList
+from .sequencing import ExecutionSequence, ConfiguredSequence
+from .action.configured import ensure_triggered
 
 logger = logging.getLogger(__name__)
 
 
-class TimePoint(object):
+class TimePoint(InstanceRegistry):
     UUID_PREFIX = 'tp-'
 
-    # timepoint instance tracking
-    _instances: List["TimePoint"] = []
-
     def __init__(self,
                  time_delta: float,
                  name: str = None,
@@ -35,6 +32,7 @@ class TimePoint(object):
         :param description: extended description for differentiation
         :param parent_sequence_id: uuid of parent SamplingScheduler (if applicable)
         """
+        InstanceRegistry.__init__(self)
         # time delta from trigger time
         self.time_delta: float = float(time_delta)
         # storage for start time
@@ -47,7 +45,6 @@ class TimePoint(object):
         self.parent_sequence_id: str = parent_sequence_id
 
         self._uuid = str(uuid.uuid4())  # assign unique identifier
-        TimePoint._instances.append(self)
         # todo add description attribute
         # todo create association between time point and parent scheduler
         # todo finish refactor
@@ -84,14 +81,12 @@ class TimePoint(object):
         :param key: key to search by
         :return: TimePoint (or subclass) instance
         """
-        for instance in cls._instances:
-            if any([
-                # instance.name == key,  # todo consider reimplementing
-                instance._uuid == key,
-                instance.uuid == key,
-            ]):
-                return instance
-        raise KeyError(f'the key "{key}" does not match an identifier for any {cls.__class__.__name__} instances')
+        warnings.warn(
+            'the get_instance method has been deprecated, use class_instance_by_id',
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return cls.class_instance_by_id(key)
 
     def as_dict(self) -> dict:
         """Returns a key: value representation of the instance"""
@@ -128,17 +123,13 @@ class TimePoint(object):
         return self.__gt__(other) or self.__eq__(other)
 
 
-class ActionTimePoint(TimePoint):
+class ActionTimePoint(TimePoint, ExecutionSequence):
     UUID_PREFIX = 'atp-'
 
     def __init__(self,
                  time_delta: float,
+                 configuration: Union[str, ConfiguredSequence],
                  name: str = None,
-                 actions: Union[
-                     List[Union[str, Callable, Action, ConfiguredAction]],
-                     ActionList,
-                     ConfiguredActionList,
-                 ] = [],
                  **action_kwargs,
                  ):
         """
@@ -158,18 +149,11 @@ class ActionTimePoint(TimePoint):
             time_delta=time_delta,
             name=name,
         )
-        # store actions for timepoint
-        self.actions = TrackedActionList(*actions)
-
-        # update kwargs
-        for action_name in action_kwargs:
-            for action in self.actions:
-                if action.name == action_name:
-                    action.update_kwargs(**action_kwargs[action_name])
-        # internal storage attributes
-        self._triggered = False
-        self._thread: threading.Thread = None
-        self._lock: threading.Lock = threading.Lock()
+        ExecutionSequence.__init__(
+            self,
+            configuration=configuration,
+            **action_kwargs,
+        )
 
     def __str__(self):
         out = f'{self.__class__.__name__} {self.string_time_delta}'
@@ -179,9 +163,6 @@ class ActionTimePoint(TimePoint):
             out += ' TRIGGERED'
         return out
 
-    def __call__(self, **kwargs):
-        self.trigger(**kwargs)
-
     def remove_action(self,
                       action: Union[str, Callable, Action],
                       ):
@@ -196,23 +177,6 @@ class ActionTimePoint(TimePoint):
         #   - determine identifier
         #   - remove from _actions list
 
-    def _execute_actions(self, **kwargs):
-        """
-        Executes the actions defined in the instance.
-
-        :param kwargs: keyword arguments for those action methods
-        """
-        execution_logger = logger.getChild(f'{self.__class__.__name__} {self.string_time_delta}')
-        with self._lock:
-            execution_logger.debug('initiating action sequence')
-            # for each action, retrieve keyword arguments if specified then execute
-            for action in self.actions:
-                action_kwargs = {}
-                if action.name in kwargs:
-                    action_kwargs.update(kwargs[action.name])
-                action(**action_kwargs)
-            execution_logger.debug('action sequence complete')
-
     def trigger(self,
                 wait: bool = None,
                 **kwargs,
@@ -231,20 +195,9 @@ class ActionTimePoint(TimePoint):
         :param kwargs: keyword arguments for the actions. Keyword arguments will be passed through to their respective
             methods.
         """
-        # todo consider refactoring to simply add a time point at current time
-        if self.triggered is True:
-            raise SystemError(f'Multiple executions of the same {self.__class__.__name__} are not supported')
-        # todo catch multiple calls and raise
-        self._triggered = True  # set flag as triggered
-        # set up thread
-        self._thread = threading.Thread(
-            target=self._execute_actions,
-            name=f'{str(self)} action thread',
-            daemon=True,
-            kwargs=kwargs,
-        )
-        self._thread.start()  # start thread
-        if wait is True:  # wait if specified
+        # todo refactor to trigger_and_wait (separate so that signatures match)
+        self.__call__(**kwargs)
+        if wait is True:
             self.wait_for_completion()
 
     @property
@@ -255,97 +208,62 @@ class ActionTimePoint(TimePoint):
         )
 
     @property
-    def triggered(self) -> bool:
-        """whether the time point has been triggered"""
-        return self._triggered
-
-    @property
-    def in_progress(self) -> bool:
-        """
-        whether the actions are in progress
-        (time point is triggered and actions are in the process of being executed)
-        """
-        if self.triggered is True:
-            return self._lock.locked()
-        return False
-
-    @property
-    def complete(self) -> bool:
-        """whether the actions have completed"""
-        if self.triggered is True:  # if instance has been triggered, check lock state
-            return not self._lock.locked()
-        return False
-
-    @property
-    def action_start_times(self) -> dict:
-        """times when the prescribed actions were started"""
-        if self.triggered is True:
-            return {
-                action.name: action.time_started for action in self.actions
-            }
-
-    @property
-    def action_completion_times(self) -> dict:
-        """times when the prescribed actions were completed"""
-        if self.triggered is True:
-            return {
-                action.name: action.time_completed for action in self.actions
-            }
-
-    @property
-    def action_durations(self) -> dict:
-        """execution durations for the prescribed actions"""
-        if self.triggered is True:
-            return {
-                action.name: action.action_duration for action in self.actions
-            }
-
-    @property
-    def started_timestamps(self) -> Dict[str, datetime.datetime]:
-        """timestamps for when actions were started"""
-        if self.triggered is True:
-            return {
-                action.name: action.started_timestamp for action in self.actions
-            }
-
-    @property
+    @ensure_triggered
     def relative_start_times(self) -> dict:
         """relative start times for actions"""
-        if self.triggered is False:
-            raise ValueError(f'the {self.__class__.__name__} has not been triggered yet')
         if self.sequence_time_started is None:
             raise ValueError(f'sequence_start_time is not set for the {self.__class__.__name__} instance')
         return {
-            action.name: action.time_started - self.sequence_time_started for action in self.actions
+            action.configuration.name: action.time_started - self.sequence_time_started for action in self.args
         }
 
-    @property
-    def action_returns(self) -> dict:
-        """retrieves the returned values from the actions. Only actions which have completed will have return values"""
-        return {
-            action.name: action.method_return for action in self.actions
-        }
-
-    @property
-    def actions_status(self) -> dict:
-        """status of the actions of the instance"""
-        return {action.name: action.status for action in self.actions}
-
-    def wait_for_completion(self, cycle_time: float = 0.1):
+    def copy_configuration(self) -> "ActionTimePoint":
         """
-        Waits for the completion of the time point
+        Copies the configuration of an ActionTimePoint instance.
 
-        :param cycle_time: cycle check time
+        :return: ActionTimePoint instance with identical configuration
         """
-        while self.in_progress is True:
-            time.sleep(cycle_time)
+        out = self.__class__(
+            time_delta=self.time_delta,
+            name=self.name,
+            configuration=self.configuration,
+        )
+        out.update_kwargs(**self.kwargs)
+        return out
+
+    @classmethod
+    def create_from_actions(cls,
+                            time_delta: float,
+                            actions: Iterable[Union[str, 'ConfiguredAction']],
+                            name: str = None,
+                            ) -> "ActionTimePoint":
+        """
+        Creates an instance from a list of configured actions
+
+        :param time_delta: time delta for the time point (relative to some other time)
+        :param actions: configured actions or named configurations
+        :param name: name for the time point (optional)
+        :return: configured instance
+        """
+        config = ConfiguredSequence(
+            *ConfiguredActionList(*actions)
+        )
+        return cls(
+            time_delta=time_delta,
+            configuration=config,
+            name=name,
+        )
 
     def as_dict(self) -> dict:
         """Returns a key: value representation of the instance"""
         out = self.basic_dict
         out['triggered'] = self.triggered
+        if self.triggered is True:
+            actions = self.args
+        else:
+            actions = self.configuration.action_sequence
         out['actions'] = [
-            action.as_dict() for action in self.actions
+            action.as_dict() for action in actions
         ]
         # todo decide how to include relative start times
         return out
